@@ -12,8 +12,11 @@
 
 from gi.repository import Gtk, GLib, WebKit2, Pango
 
+from gettext import gettext as _
+
 from eolie.label_indicator import LabelIndicator
 from eolie.define import App, ArtSize
+from eolie.utils import resize_favicon
 
 
 class PagesManagerChild(Gtk.FlowBoxChild):
@@ -30,13 +33,12 @@ class PagesManagerChild(Gtk.FlowBoxChild):
         Gtk.FlowBoxChild.__init__(self)
         self.__view = view
         self.__window = window
-        self.__favicon = None
         self.__connected_ids = []
         self.__scroll_timeout_id = None
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Eolie/PagesManagerChild.ui")
         builder.connect_signals(self)
-        self.__indicator_label = LabelIndicator()
+        self.__indicator_label = LabelIndicator(False)
         self.__indicator_label.mark_unshown(view.webview)
         self.__indicator_label.set_hexpand(True)
         self.__indicator_label.set_margin_right(4)
@@ -46,8 +48,10 @@ class PagesManagerChild(Gtk.FlowBoxChild):
         self.__indicator_label.show()
         if view.webview.title:
             self.__indicator_label.set_text(view.webview.title)
-        builder.get_object("grid").attach(self.__indicator_label, 0, 0, 1, 1)
+        builder.get_object("grid").attach(self.__indicator_label, 1, 0, 1, 1)
         self.__image = builder.get_object("image")
+        self.__pin_button = builder.get_object("pin_button")
+        self.__pin_image = builder.get_object("pin_image")
         self.__close_button = builder.get_object("close_button")
         self.__close_button.get_image().set_from_icon_name(
             "window-close-symbolic",
@@ -73,7 +77,7 @@ class PagesManagerChild(Gtk.FlowBoxChild):
         view.connect("destroying", self.__on_view_destroying)
         self.__view.webview.connect("snapshot-changed",
                                     self.__on_webview_snapshot_changed)
-        self.__view.webview.connect("favicon-changed",
+        self.__view.webview.connect("notify::favicon",
                                     self.__on_webview_favicon_changed)
         self.__view.webview.connect("notify::is-playing-audio",
                                     self.__on_webview_notify_is_playing_audio)
@@ -124,6 +128,19 @@ class PagesManagerChild(Gtk.FlowBoxChild):
         """
         pass
 
+    def _on_pin_button_clicked(self, button):
+        """
+            Pin/Unpin page
+            @param button as Gtk.Button
+        """
+        if self.view.webview.uri in App().pinned:
+            self.__pin_image.set_opacity(0.5)
+            App().remove_from_pinned(self.view.webview.uri)
+        else:
+            self.__pin_image.set_opacity(1)
+            App().add_to_pinned(self.view.webview.uri)
+        return True
+
     def _on_close_button_clicked(self, button):
         """
             Destroy self
@@ -138,6 +155,13 @@ class PagesManagerChild(Gtk.FlowBoxChild):
             @param eventbox as Gtk.EventBox
             @param event as Gdk.Event
         """
+        if self.view.webview.uri in App().pinned:
+            self.__pin_image.set_opacity(1)
+            self.__pin_button.set_tooltip_text(_("Unpin this page"))
+        else:
+            self.__pin_image.set_opacity(0.5)
+            self.__pin_button.set_tooltip_text(_("Pin this page"))
+        self.__pin_button.set_opacity(1)
         self.__close_button.get_image().set_from_icon_name(
             "window-close-symbolic",
             Gtk.IconSize.INVALID)
@@ -153,6 +177,7 @@ class PagesManagerChild(Gtk.FlowBoxChild):
            event.x >= allocation.width or\
            event.y <= 0 or\
            event.y >= allocation.height:
+            self.__pin_button.set_opacity(0)
             self.__on_webview_favicon_changed(self.__view.webview)
 
 #######################
@@ -217,34 +242,29 @@ class PagesManagerChild(Gtk.FlowBoxChild):
         """
         self.__on_webview_favicon_changed(webview)
 
-    def __on_webview_favicon_changed(self, webview, surface=None):
+    def __on_webview_favicon_changed(self, webview, *ignore):
         """
             Set favicon
             @param webview as WebView
-            @param surface as cairo.Surface
         """
+        if webview.current_event == WebKit2.LoadEvent.STARTED:
+            return
         image = self.__close_button.get_image()
         if webview.is_playing_audio():
             image.set_from_icon_name("audio-speakers-symbolic",
                                      Gtk.IconSize.INVALID)
             return
-
-        if surface is not None:
-            image.set_from_surface(surface)
-            return
-
-        favicon_path = App().art.get_favicon_path(webview.uri)
-        if favicon_path is not None:
-            image.set_from_file(favicon_path)
-            return
-
         artwork = App().art.get_icon_theme_artwork(webview.uri,
                                                    webview.ephemeral)
         if artwork is not None:
             image.set_from_icon_name(artwork, Gtk.IconSize.INVALID)
-            return
-
-        image.set_from_icon_name("applications-internet", Gtk.IconSize.INVALID)
+        else:
+            surface = webview.get_favicon()
+            if surface is not None:
+                image.set_from_surface(resize_favicon(surface))
+            else:
+                image.set_from_icon_name("applications-internet",
+                                         Gtk.IconSize.INVALID)
 
     def __on_webview_title_changed(self, webview, title):
         """
@@ -262,13 +282,15 @@ class PagesManagerChild(Gtk.FlowBoxChild):
         """
         uri = webview.uri
         if event == WebKit2.LoadEvent.STARTED:
-            self.__favicon = None
-            self.__image.clear()
+            image = self.__close_button.get_image()
+            image.set_from_icon_name("content-loading-symbolic",
+                                     Gtk.IconSize.INVALID)
             self.__spinner.start()
             self.__indicator_label.set_text(uri)
         elif event == WebKit2.LoadEvent.COMMITTED:
             self.__indicator_label.set_text(uri)
         elif event == WebKit2.LoadEvent.FINISHED:
+            self.__on_webview_favicon_changed(webview)
             self.__spinner.stop()
 
     def __on_webview_snapshot_changed(self, webview, surface):
